#!/bin/bash

client_dir="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
export client_dir

cleanup() {
    [ ! -z $validation_log ] && [ -f $validation_log ] && rm -f $validation_log
    [ ! -z $response ] && [ -f $response ] && rm -f $response
}
trap 'cleanup; exit 1' SIGINT SIGTERM

err_quit() {
    echo $1 1>&2
    exit 1
}

if [ ! -f "$client_dir/client.cfg" ]; then
    err_quit "'$client_dir/client.cfg' doesn't exist, create it first"
fi
. "$client_dir/client.cfg"

check_file() {
    [ -z "$1" ] && err_quit "Filename is empty"
    [ ! -e "$1" ] && err_quit "File '$1' does not exist"
    [ ! -f "$1" ] && err_quit "File '$1' is not a file"
    [ ! -r "$1" ] && err_quit "File '$1' is not readable"
}

usage() {
    cat <<EOF
client.sh COMMAND [ OPTIONS ]

client.sh check_out TASKID

Metadata Generation:
client.sh check_in TASKID -h HardwareDescription 
  -m MetdataStoreInfo

Semantic Query Generation:
client.sh check_in TASKID -h HardwareDescription 
  -s SemanticQuery

Event Query Generation:
client.sh  check_in TASKID -h HardwareDescription 
  -e EventQuery

Event Search:
client.sh  check_in TASKID -h HardwareDescription 
  -d Detections -t Threshold [ -r Recountings ]

System Description:
client.sh up_sysdesc -S SystemDescription

COMMANDS
  next                    Fetch the list of tasks that can be started.  Server 
                          returns a csv file giving a list of tasks (TaskIDs),
                          their optionality and open/close dates.

  outstanding             Fetch the list of tasks which have been checked out.

  all                     Fetch the list of all tasks already completed or in
                          progress.

  up_sysdesc              Upload system description.

  rerun_ps_mg             Set flag to allow re-generation of Event-BG Metadata 
                          Store and PS-Training Metadata Store.

  rerun_ps_training_mg    Set flag to allow re-generation of PS-Training 
                          Metadata Store.

  reset_dr_tasks          Resets all Dry Run tasks (including Metadata 
                          Generation for MED14-Test) on the server.

  check_out               Check out resources for a task and set the start
                          time for that task.  If task is successfully checked
                          out, this script returns set of key:value pairs for
                          each task input file. 

  check_in                Check in module output for a task that has been
                          checked out.  Module output is validated prior to
                          submission.  Validation can be performed without
                          submission by using the -V option

EOF
}

check_cfg() {
    if [[ $prf != "PRF" && $prf != "noPRF" ]]; then
	err_quit "Configuration error: variable 'prf' should be set to either \"PRF\" or \"noPRF\""
    fi
    if [[ $mer != "Yes" && $mer != "No" ]]; then
	err_quit "Configuration error: variable 'mer' should be set to either \"Yes\" or \"No\""
    fi
    if [[ $med_eval_part != "Full" && $med_eval_part != "Sub" ]]; then
	err_quit "Configuration error: variable 'med_eval_part' should be set to either \"Full\" or \"Sub\""
    fi
    [ -z $working_dir ] && err_quit "Configuration error: variable 'working_dir' is unset or empty"
    if [ ! -d $working_dir ]; then
	err_quit "Configuration error: working_dir '$working_dir' doesn't exist or isn't a directory"
    fi
    [ -z $schema_dir ] && err_quit "Configuration error: variable 'schema_dir' is unset or empty"
    if [ ! -d $schema_dir ]; then
	err_quit "Configuration error: schema_dir '$schema_dir' doesn't exist or isn't a directory"
    fi
}
check_cfg

module=''
evaluation=''
search_set=''
event=''
split_task_id() {
    module=`cut -d _ -f 1 <<<$1`
    if [ $module != 'MG' ]; then
	evaluation=$module
	module=`cut -d _ -f 2 <<<$1`
    fi
    # Parse extra fields for ES validation
    if [ $module = 'ES' ]; then
	search_set="MED14-Eval$med_eval_part"
	if [ $evaluation = 'DR' ]; then
	    search_set="MED14-Test"
	fi
	event=`cut -d _ -f 3 <<<$1`
    fi
}

check_task_id() {
    if [ -z $task_id ]; then
	err_quit 'Missing required TaskID'
    fi
    if [ ${task_id:0:1} = '-' ]; then
	err_quit "TaskID '$task_id' looks like an option instead of a TaskID, check the usage"
    fi
    if [[ $cmd = "check_out" && ${task_id:(-4)} = '_PRF' ]]; then
	err_quit "Cannot explicitly check out PRF tasks"
    fi
}

cmd=$1
shift

case $cmd in
    check_out | check_in)
	route=$cmd
	task_id=$1
	check_task_id
	split_task_id $task_id
	params="task_id=${task_id}&prf=${prf}&mer=${mer}&part=${med_eval_part}"
	shift
	;;
    next | outstanding | all | rerun_ps_mg | rerun_ps_training_mg | up_sysdesc | reset_dr_tasks )
	route=$cmd
	;;
    "help" | "")
	usage
	exit 0
	;;
    *)
	err_quit "Unrecognized command '$cmd', use the command 'help' for usage."
esac

only_validate=0
while getopts "h:m:s:e:d:t:r:D:T:R:S:V" opt
do
    case $opt in
	h)
	    hw_desc=$OPTARG
	    check_file $hw_desc
	    files="${files} -F hw_desc=@${hw_desc}"
	    shift $((OPTIND-1)); OPTIND=1
	    ;;
	m)
	    ms_info=$OPTARG
	    check_file $ms_info
	    files="${files} -F ms_info=@${ms_info}"
	    shift $((OPTIND-1)); OPTIND=1
	    ;;
	s)
	    sq=$OPTARG
	    check_file $sq
	    files="${files} -F sq=@${sq}"
	    shift $((OPTIND-1)); OPTIND=1
	    ;;
	e)
	    eq=$OPTARG
	    check_file $eq
	    files="${files} -F eq=@${eq}"
	    shift $((OPTIND-1)); OPTIND=1
	    ;;
	d)
	    det=$OPTARG
	    check_file $det
	    files="${files} -F det=@${det}"
	    shift $((OPTIND-1)); OPTIND=1
	    ;;
	t)
	    thresh=$OPTARG
	    check_file $thresh
	    files="${files} -F thresh=@${thresh}"
	    shift $((OPTIND-1)); OPTIND=1
	    ;;
	r)
	    recountings=$OPTARG
	    check_file $recountings
	    files="${files} -F mer=@${recountings}"
	    shift $((OPTIND-1)); OPTIND=1
	    ;;
	S)
	    sysdesc=$OPTARG
	    check_file $sysdesc
	    files="${files} -F sysdesc=@${sysdesc}"
	    shift $((OPTIND-1)); OPTIND=1
	    ;;
	V)
	    only_validate=1
	    shift $((OPTIND-1)); OPTIND=1
	    ;;
	?)
	    err_quit "Invalid option '$opt'"
	    ;;
    esac
done

#Validation
if [ $route = 'check_in' ]; then
    case $module in
	MG)
	    [ -z $hw_desc ] && err_quit "Missing required option -h HardwareDescription"
	    [ -z $ms_info ] && err_quit "Missing required option -m MetadataStoreInfo"
	    validation_log=`mktemp -t med_io_validation_XXXXX`
	    bash "$client_dir/lib/validate_mg_in.sh" "$client_dir/client.cfg" $ms_info $hw_desc 2>$validation_log
	    [ $? != 0 ] && err_quit "Error during local validation, check validation log '$validation_log'"
	    ;;
	SQG)
	    [ -z $hw_desc ] && err_quit "Missing required option -h HardwareDescription"
	    [ -z $sq ] && err_quit "Missing required option -s SemanticQuery"
	    validation_log=`mktemp -t med_io_validation_XXXXX`
	    bash "$client_dir/lib/validate_sqg_in.sh" "$client_dir/client.cfg" $sq $hw_desc 2>$validation_log
	    [ $? != 0 ] && err_quit "Error during local validation, check validation log '$validation_log'"
	    ;;
	EQG)
	    [ -z $hw_desc ] && err_quit "Missing required option -h HardwareDescription"
	    [ -z $eq ] && err_quit "Missing required option -e EventQuery"
	    validation_log=`mktemp -t med_io_validation_XXXXX`
	    bash "$client_dir/lib/validate_eqg_in.sh" "$client_dir/client.cfg" $eq $hw_desc 2>$validation_log
	    [ $? != 0 ] && err_quit "Error during local validation, check validation log '$validation_log'"
	    ;;
	ES)
	    [ -z $hw_desc ] && err_quit "Missing required option -h HardwareDescription"
	    [ -z $det ] && err_quit "Missing required option -d Detections"
	    [ -z $thresh ] && err_quit "Missing required option -t Threshold"
	    [ $mer = "Yes" ] && [[ $task_id = *010Ex || $task_id = *010Ex_PRF ]] && [ -z $recountings ] && err_quit "Missing required option -r Recountings"
	    validation_log=`mktemp -t med_io_validation_XXXXX`
	    bash "$client_dir/lib/validate_es_in.sh" "$client_dir/client.cfg" $search_set $event $det $thresh $recountings $hw_desc 2>$validation_log
	    [ $? != 0 ] && err_quit "Error during local validation, check validation log '$validation_log'"
	    ;;
	*)
	    err_quit "TaskID '$task_id' appears to be malformed, aborting!"
	    ;;
    esac
    [ -f $validation_log ] && rm -f $validation_log
elif [ $route = 'up_sysdesc' ]; then
    [ -z $sysdesc ] && err_quit "Missing required option -S System Description"
fi

# Finish if only validating
if [ $route = "check_in" ] && [ $only_validate -eq 1 ]; then
    echo "Validation successful"
    exit 0
fi

# echo [curl --cert $user_certificate --key $user_key --cacert $ca_certificate $files "${server_url}/${route}?${params}"]
response=`mktemp -t med_io_response_XXXXX`
curl_err=`mktemp -t med_io_curl_err_XXXXX`
response_code=`curl -s -S -w "%{response_code}" -o $response --cert $user_certificate --key $user_key --cacert $ca_certificate $files "${server_url}/${route}?${params}" 2>${curl_err}`
curl_exit=$?

if [ $curl_exit -eq 35 ]; then
    cat <<EOF
curl encountered an error trying to connect to the server.  This is
likely due to the I/O Server being down or a client configuration
issue.  Contact NIST at <med_poc@nist.gov> and include the following
information in your email:

EOF
    if [ -n `which openssl` ]; then
	echo -n "\"`openssl x509 -subject -noout -in ${user_certificate} | sed 's:^.*/CN=\([a-zA-Z\-_]*\)$:\1:'`\","
    fi
    echo -n "\"${cmd}\"",
    [ "$task_id" != "" ] && echo -n "\"${task_id}\"",
    echo "\"`date`\""
    [ -f $curl_err ] && cat $curl_err && rm -f $curl_err
    exit 1
fi

correct_path() {
    sed 's%^\./%'$working_dir'/%'
}

case $response_code in
    000)
	[ -f $response ] && rm -f $response
	[ -f $curl_err ] && cat $curl_err && rm -f $curl_err
	exit $curl_exit
	;;
    4**)
	cat $response
	[ -f $response ] && rm -f $response
	[ -f $curl_err ] && rm -f $curl_err
	exit 1
	;;
    200)
	case $route in
	    check_in)
		sed 's/^\(.*\)$/TimeTaken:\1/' $response 
		;;
	    check_out)
		tar -xzf $response -C $working_dir
		case $module in
		    MG)
			tar -tzf $response | correct_path | while read file; do
			    [[ $file = *.in.name ]] && echo InMS:"\"`cat $file`\""
			    [[ $file = *.out.name ]] && echo OutMS:"\"`cat $file`\""
			    [[ $file = *.csv ]] && echo video-set:$file
			    [[ $file = *.pw ]] && echo PW:"\"`cat $file`\""
			done
			;;
		    SQG)
			tar -tzf $response | correct_path | while read file; do
			    if [[ $file = *E[0-9]*.txt ]]; then
				echo ${file/*/EventDescription:$file}
				base=`basename $file`
				echo eventID:"\"${base%.txt}\""
			    fi
			done
			;;
		    EQG)
			tar -tzf $response | correct_path | while read file; do
			    [[ $file = *SQ.xml ]] && echo SemanticQuery:$file
			    [[ $file = *.csv ]] && echo ExemplarLabels:$file
			    [[ $file = *.out.name ]] && echo TrainMS:"\"`cat $file`\""
			done
			;;
		    ES)
			tar -tzf $response | correct_path | while read file; do
			    [[ $file = *.xml ]] && echo Query:$file
			    [[ $file = *.out.name ]] && echo SearchMS:"\"`cat $file`\""
			done
			echo PRF:"\"$prf\""
			;;
		esac
		;;
	    next)
		# Suppress PRF tasks
		sed '/_PRF,/d' $response
		;;
	    outstanding | all)
		cat $response
		;;
	    *)
		sed 's/^\(.*\)$/Response:"\1"/' $response
	    ;;
	esac
	;;
esac
[ -f $response ] && rm -f $response
[ -f $curl_err ] && rm -f $curl_err

exit 0
